uniform float time;         // bazik preset playback time in seconds
uniform vec2 resolution;    // bazik render width (x) & height (y)

uniform vec3 color1;        // bazik user color 1 (r=x, g=y, b=z)
uniform vec3 color2;        // bazik user color 2 (r=x, g=y, b=z)
uniform vec3 color3;        // bazik user color 3 (r=x, g=y, b=z)
uniform vec3 color4;        // bazik user color 4 (r=x, g=y, b=z)

uniform float eq1;          // bazik 3 band equalizer bass value 0.0 to 1.0
uniform float eq2;          // bazik 3 band equalizer medium value 0.0 to 1.0
uniform float eq3;          // bazik 3 band equalizer high value 0.0 to 1.0
uniform float banger;       // bazik music banger (beat detect) 0.0 to 1.0

uniform int breaker;        // bazik user control 0 or 1
uniform int mode;           // bazik old user control 1,2,3,4,5,6,7 or 8
uniform float shift;        // bazik user shift random value 0.0 to 1.0

uniform float par1;         // bazik user control -1.0 to 1.0
uniform float par2;         // bazik user control -1.0 to 1.0


const float pi = 3.14159;

mat3 xrot(float t)
{
    return mat3(1.0, 0.0, 0.0,
                0.0, cos(t), -sin(t),
                0.0, sin(t), cos(t));
}

mat3 yrot(float t)
{
    return mat3(cos(t), 0.0, -sin(t),
                0.0, 1.0, 0.0,
                sin(t), 0.0, cos(t));
}

mat3 zrot(float t)
{
    return mat3(cos(t), -sin(t), 0.0,
                sin(t), cos(t), 0.0,
                0.0, 0.0, 1.0);
}

float box(vec3 p, vec3 s)
{
	vec3 q = clamp(p, -s, s);
    return length(p - q);
}

float wscl = 6.0;

float map(vec3 p)
{
    float h = wscl * 2.0;
	vec3 g = (fract((p+vec3(h*0.5))/h) * 2.0 - 1.0) * h;
    vec3 q = g;
    for (int i = 0; i < 2; ++i) {
        vec3 r = sign(q);
        q = r + reflect(q - r, normalize(r));
        vec3 aq = abs(q);
        q -= sign(q) * min(aq.x, min(aq.y, aq.z)) * ((0.5 * 3.0 + shift)*0.25);
    }
    float d = box(q, vec3(1.0));
    float tr = 0.5 + par2 * sin(p.z*0.5);
    tr = mix(2.5, 6.0, tr);
    float c = length(g.xy) - tr;
    float e = length(g.xz) - tr;
    float f = length(g.yz) - tr;
    float cv = min(min(c, e), f);
    return max(d,-cv);
}

vec3 normal(vec3 p)
{
	vec3 o = vec3(0.01, 0.0, 0.0);
    return normalize(vec3(map(p+o.xyy) - map(p-o.xyy),
                          map(p+o.yxy) - map(p-o.yxy),
                          map(p+o.yyx) - map(p-o.yyx)));
}

float trace(vec3 o, vec3 r)
{
    float t = 0.0;
    for (int i = 0; i < 42; ++i) {
        vec3 p = o + r * t;
        float d = map(p);
        t += d * 0.35;
    }
    return t;
}

uniform sampler2D myTexture;

vec3 texture(vec3 p)
{
	vec3 ta = color1 * (0.5 + eq1*0.2);
    vec3 tb = texture2D(myTexture, vec2(p.x,p.z)).xyz;
    vec3 tc = texture2D(myTexture, vec2(p.x,p.y)).xyz;
    return (ta + tb + tc) / 3.0;
}

vec3 shade(vec3 o, vec3 r, vec3 w, vec3 sn, float t)
{
    float fd = map(w);
    
    float fog = 1.0 / (1.0 + t * t * 0.01 + fd * 100.0);
    vec3 diff = texture(w);
    
    vec3 lit = vec3(0.5);
    vec3 spec = vec3(0.0);
    
    for (int i = -1; i <= 1; i+=2) {
        float j = float(i) * 0.5 + 0.5;

        vec3 loff = color1 * sign(float(i));

        vec3 ldir = normalize(o + loff - w);

        vec3 lcol = mix(color2, color3, j);
        
        lit += max(dot(sn,ldir), 0.0) * lcol;

        vec3 refl = reflect(ldir, -sn);
        float spr = clamp(dot(refl, r), 0.0, 1.0);
        spr = pow(spr, 4.0);
        
        spec += lcol * spr;
    }

    diff *= lit;
    diff += spec * 0.5;
    
    if(breaker == 1)diff *= 0.4;

    return diff * fog;
}

void main(void)
{
	vec2 uv = gl_FragCoord.xy.xy / resolution.xy;
    uv = uv * 2.0 - 1.0;
    uv.x *= resolution.x / resolution.y;
    
    vec3 o = vec3(0.0, 0.0, time*2.0 + eq1*0.2);
    
    if(breaker == 1) o = vec3(0.0, 0.0, time*2.0);

    vec3 r = normalize(vec3(uv, 0.7 + par1));
    r *= zrot(time*(shift - 0.25));
    
    float t = trace(o, r);
    vec3 w = o + r * t;
    vec3 sn = normal(w);
	vec3 sc = shade(o, r, w, sn, t);
    
	gl_FragColor = vec4(sqrt(sc), 1.0);
}