uniform float time;         // bazik preset playback time in seconds
uniform vec2 resolution;    // bazik render width (x) & height (y)

uniform vec3 color1;        // bazik user color 1 (r=x, g=y, b=z)
uniform vec3 color2;        // bazik user color 2 (r=x, g=y, b=z)
uniform vec3 color3;        // bazik user color 3 (r=x, g=y, b=z)
uniform vec3 color4;        // bazik user color 4 (r=x, g=y, b=z)

uniform float eq1;          // bazik 3 band equalizer bass value 0.0 to 1.0
uniform float eq2;          // bazik 3 band equalizer medium value 0.0 to 1.0
uniform float eq3;          // bazik 3 band equalizer high value 0.0 to 1.0
uniform float banger;       // bazik music banger (beat detect) 0.0 to 1.0

uniform int breaker;        // bazik user control 0 or 1
uniform int mode;           // bazik old user control 1,2,3,4,5,6,7 or 8
uniform float shift;        // bazik user shift random value 0.0 to 1.0

uniform float par1;         // bazik user control -1.0 to 1.0
uniform float par2;         // bazik user control -1.0 to 1.0


float map(vec3 p) {
    vec3 q = fract(p) * 2.0 - 1.0;
    
	return length(q) - 0.1;
}

float trace(vec3 o, vec3 r) {
	float t = 0.0;
    for (int i = 0; i < 6; i++) {
    	vec3 p = o + r *t;
        float d = map(p);
        t += d * (1.5 - par1);
    }
    return t;
}

void main(void)
{
	vec2 uv = gl_FragCoord.xy / resolution.xy;
    
    uv = uv * 2.0 - ((par2*1.0) + 1.0);
    
    uv.x *= resolution.x / resolution.y;
    
    float t = time * shift;
    
    if(shift < 0.4)t = time * - shift;
    
    
    uv *= mat2(cos(t), -sin(t), sin(t), cos(t));
    
    vec3 r = normalize(vec3(uv, 4.0 * shift));
    
    float vr = eq1*0.2 + time * 0.4;
    if(breaker == 1) vr = 0.0 + eq2*0.8;

    vec3 o = vec3(1.0, 0.0, vr);
    
    float tr = trace(o, r);
    
    float fog = 1.0 / (1.0 + tr * tr * 0.1);
    
    vec3 fc = vec3(fog);
    
	gl_FragColor = vec4(fc, 1.0);
}